from nose.tools import *
import os
import tempfile
from scr_dim.Config import ConfigBase


def test_truncate():
  c = ConfigBase()
  assert_almost_equal(c.max_brightness, c._truncate_brightness(c.max_brightness + 1.0))
  assert_almost_equal(c.min_brightness, c._truncate_brightness(c.min_brightness - 1.0))



import os
import argparse
import re
import operator
from scr_dim.ProcLock import ProcLock
from scr_dim.Config import Config
from scr_dim.Executor import Executor

script_dir = os.path.realpath(os.path.dirname(__file__))


class ScrDim(object):
  """core logic to manipulate screen brightness"""

  def __init__(self, conf=Config(), cmd=Executor()):
    self.conf = conf
    self.cmd= cmd

  def _set_brightness(self, val):
    self.conf.brightness = val
    # do not pass val arg to cmd since brightness got possibly truncated by conf
    return all(self.cmd.set_brightness(t, self.conf.brightness) \
        for t in self.cmd.connected_outputs())

  def _mod_brightness(self, op, *val):
    return self._set_brightness(op(self.conf.brightness, *val))

  @property
  def command_names(self):
    if not hasattr(self, '_cmd_names') or not self._cmd_names:
      cmd_re = re.compile('^cmd_(.*)$')
      self._cmd_names = [
          m.group(1) for m in filter(None, map(cmd_re.match, dir(self)))]
    return self._cmd_names

  def get_cmd(self, cmd):
    return getattr(self, 'cmd_%s' % cmd)

  def cmd_inc(self, steps=1, *_):
    return all([self._mod_brightness(self.conf.op, *self.conf.inc_coeff) \
        for _ in range(int(steps))])

  def cmd_dec(self, steps=1, *_):
    return all([self._mod_brightness(self.conf.op, *self.conf.dec_coeff) \
        for _ in range(int(steps))])

  def cmd_reset(self, force_icc=None, *_):
    res = self._set_brightness(1.0)
    if force_icc or self.conf.icc:
      # going to ignore set_brightness return value
      if force_icc and os.path.isfile(force_icc):
        icc_path = force_icc
      else:
        icc_path = self.conf.icc_path
      # let it fail if the path is invalid
      return self.cmd.load_icc(self.conf.icc_path)
    else:
      return res

  def cmd_set(self, val=None, *_):
    val = float(val) if val else self.conf.brightness
    return self._set_brightness(val)


class Runner(ProcLock):
  """provide command line entry point"""

  def __init__(self, conf=Config()):
    self.core = ScrDim(conf=conf)
    super().__init__(os.path.join(conf.conf_dir, '.lockfile'))

  def main(self):
    p = argparse.ArgumentParser()
    p.add_argument('cmd', choices=self.core.command_names)
    p.add_argument('cmd_args', nargs='*')
    args = p.parse_args()
    return 0 if self.core.get_cmd(args.cmd)(*args.cmd_args) else -1



from nose.tools import *
from nose.exc import SkipTest
import shutil
import os
import tempfile
from scr_dim.Config import Config

# safe config directory path for test
tmp_confdir = os.path.join(tempfile.gettempprefix(), tempfile.gettempprefix() + 'scr_dim_test_dir')


def setup_cleandir():
  if not shutil.rmtree.avoids_symlink_attacks:
    raise SkipTest('platform does not support symlink-attack-free rmtree')
  if os.path.exists(tmp_confdir):
    shutil.rmtree(tmp_confdir)


@with_setup(setup_cleandir)
def test_dir_init():
  c = Config(conf_dir=tmp_confdir)
  eq_(tmp_confdir, c.conf_dir)
  ok_(os.path.isdir(c.conf_dir))


@with_setup(setup_cleandir)
def test_file_init():
  c = Config(conf_dir=tmp_confdir)
  # safety assert so that test doesn't purge default real config dir
  eq_(tmp_confdir, c.conf_dir)
  eq_(os.path.join(tmp_confdir, 'config'), c.conf_path)
  ok_(os.path.isfile(c.conf_path))


@with_setup(setup_cleandir)
def test_default_value():
  c = Config(conf_dir=tmp_confdir)
  eq_(tmp_confdir, c.conf_dir)  # safety assert
  assert_almost_equal(1.0, c['brightness'])
  eq_('default.icc', c[c.k_icc_path])
  ok_(not c[c.k_icc])


@with_setup(setup_cleandir)
def test_save_load():
  c1 = Config(conf_dir=tmp_confdir)
  eq_(tmp_confdir, c1.conf_dir)  # safety assert
  c1.update({
    'brightness': 0.5,
    c1.k_icc_path: 'alternate.icc',
  })
  c2 = Config(conf_dir=tmp_confdir)
  eq_(tmp_confdir, c2.conf_dir)  # safety assert
  assert_almost_equal(0.5, c2['brightness'])
  eq_('alternate.icc', c2[c2.k_icc_path])


@with_setup(setup_cleandir)
def test_partial_update():
  c = Config(conf_dir=tmp_confdir)
  eq_(tmp_confdir, c.conf_dir)  # safety assert
  c.update({
    c.k_value: 0.7,
    c.k_icc_path: 'my_profile.icc',
  })
  c.update({c.k_value: 0.5})
  assert_almost_equal(0.5, c[c.k_value])
  eq_('my_profile.icc', c[c.k_icc_path])
  c.update({c.k_icc_path: 'fallback_profile.icc'})
  assert_almost_equal(0.5, c[c.k_value])
  eq_('fallback_profile.icc', c[c.k_icc_path])


@with_setup(setup_cleandir)
def test_icc_abspath():
  c = Config(conf_dir=tmp_confdir)
  eq_(tmp_confdir, c.conf_dir)  # safety assert
  abspath = os.path.abspath(os.path.join(tempfile.gettempdir(), tempfile.gettempprefix() + 'alternate.icc'))
  c.update({ c.k_icc_path: abspath })
  eq_(abspath, c.icc_path)


@with_setup(setup_cleandir)
def test_icc_relpath():
  c = Config(conf_dir=tmp_confdir)
  eq_(tmp_confdir, c.conf_dir)  # safety assert
  relpath = os.path.join('my_icc_profiles', '..', 'prof.icc')
  c.update({ c.k_icc_path: relpath })
  eq_(os.path.abspath(os.path.join(c.conf_dir, relpath)), c.icc_path)



import os
import json
import operator


class ConfigBase(object):
  k_min = 'brightness_max'
  k_max = 'brightness_min'
  k_op = 'step_operator'
  k_inc = 'step_coeff_inc'
  k_dec = 'step_coeff_dec'
  k_value = 'brightness'
  k_icc = 'icc_profile_load_on_reset'
  k_icc_path = 'icc_profile_path'

  def __init__(self):
    # TODO: validate config content for versioning
    self._conf = self._loadconf()
    if not self._conf:
      self._defaultconf()

  def _saveconf(self):
    pass

  def _loadconf(self):
    return {}

  def __getitem__(self, k):
    return self._conf[k]

  def update(self, dic):
    self._conf.update(dic)

  def _truncate_brightness(self, val):
    return min(max(val, self.min_brightness), self.max_brightness)

  @property
  def max_brightness(self):
    return self[self.k_max]

  @property
  def min_brightness(self):
    return self[self.k_min]

  @property
  def brightness(self):
    return self[self.k_value]

  @property
  def op(self):
    return getattr(operator, self[self.k_op])

  @property
  def inc_coeff(self):
    return self[self.k_inc]

  @property
  def dec_coeff(self):
    return self[self.k_dec]

  @brightness.setter
  def brightness(self, val):
    val = self._truncate_brightness(val)
    self.update({self.k_value: val})

  def _defaultconf(self):
    self._conf = {
        self.k_max: 1.0,
        self.k_min: 0.05,
        self.k_op: 'mul',
        self.k_inc: [1.08],
        self.k_dec: [0.926],
        self.k_value: 1.0,
        self.k_icc: False,
        self.k_icc_path: 'default.icc',
    }
    self._saveconf()


class Config(ConfigBase):
  """retrieves config values from filesystem"""

  def __init__(self, **kwargs):
    self.conf_dir = kwargs.get('conf_dir', \
        os.path.join(os.path.expanduser('~'), '.scr_dim'))
    self.conf_path = os.path.join(self.conf_dir, 'config')
    super().__init__()

  def _ensure_confdir(self):
    if not os.path.exists(self.conf_dir):
      try:
        os.makedirs(self.conf_dir)
      except IOError:
        pass
    if not os.path.isdir(self.conf_dir):
      raise Error('cannot create config directory at %s' \
          % self.conf_dir)

  def _loadconf(self):
    self._ensure_confdir()
    try:
      with open(self.conf_path, 'r') as fp:
        return json.load(fp)
    except:
      return None

  def _saveconf(self):
    self._ensure_confdir()
    with open(self.conf_path, 'w') as fp:
      enc = json.JSONEncoder(indent=2, separators=(',', ': '), sort_keys=True)
      fp.write(enc.encode(self._conf))

  def update(self, dic):
    self._conf.update(dic)
    self._saveconf()

  @property
  def icc(self):
    return self[self.k_icc]

  @property
  def icc_path(self):
    p = self[self.k_icc_path]
    if os.path.isdir(p):
      return p
    else:
      return os.path.abspath(os.path.join(self.conf_dir, p))




import os
import fcntl
import threading

class ProcLock(object):
  """ makes sure that no parallel execution occurs"""
  # in-process lock
  _cache_lock = threading.Lock()
  # list of acquired inter-process lockfile
  _cache = []

  def __init__(self, path):
    self.path = os.path.abspath(path)
    with self._cache_lock:
      if self.path in self._cache:
        raise Exception("cannot lock same file twice")
      self.fp = open(self.path, 'w')
      fcntl.lockf(self.fp, fcntl.LOCK_EX | fcntl.LOCK_NB)
      self._cache.append(self.path)

  def __enter__(self):
    return self

  def release(self):
    """releases lock. safe to call multiple times"""
    with self._cache_lock:
      if not self.path in self._cache:
        return
      os.remove(self.path)
      fcntl.lockf(self.fp, fcntl.LOCK_UN)
      self.fp.close()
      self._cache.remove(self.path)

  def __exit__(self, exc_type, exc_val, exc_tb):
    self.release()

  def __del__(self):
    self.release()



from nose.tools import *
import os
import tempfile
import collections
from scr_dim.core import ScrDim
from scr_dim.Config import ConfigBase

class DummyExecutor(object):
  def __init__(self):
    self.history = collections.defaultdict(lambda: [])

  def _add_record(self, cmd, *args):
    self.history[cmd].append(args)

  def connected_outputs(self):
    return ['out1', 'out2']

  def set_brightness(self, *args):
    self._add_record('set_brightness', *args)
    return True

  def load_icc(self, *args):
    self._add_record('load_ic', *args)
    return True


class DummyConfig(ConfigBase):
  def __init__(self):
    super().__init__()
    self.brightness = 0.5


def test_set_current():
  c = ScrDim(conf=DummyConfig(), cmd=DummyExecutor())
  nouts = len(c.cmd.connected_outputs())
  expected = c.conf.brightness
  res = c.get_cmd('set')()
  hist = c.cmd.history['set_brightness']
  ok_(res)
  eq_(nouts, len(hist))
  for n in range(nouts):
    eq_(hist[n][1], expected)


def test_set():
  c = ScrDim(conf=DummyConfig(), cmd=DummyExecutor())
  nouts = len(c.cmd.connected_outputs())
  expected = 0.88
  res = c.get_cmd('set')(expected)
  hist = c.cmd.history['set_brightness']
  ok_(res)
  eq_(nouts, len(hist))
  for n in range(nouts):
    eq_(hist[n][1], expected)


def test_inc():
  c = ScrDim(conf=DummyConfig(), cmd=DummyExecutor())
  nouts = len(c.cmd.connected_outputs())
  expected = c.conf.brightness * c.conf.inc_coeff[0]
  res = c.get_cmd('inc')()
  hist = c.cmd.history['set_brightness']
  ok_(res)
  eq_(nouts, len(hist))
  for n in range(nouts):
    eq_(hist[n][1], expected)


def test_dec():
  c = ScrDim(conf=DummyConfig(), cmd=DummyExecutor())
  nouts = len(c.cmd.connected_outputs())
  expected = c.conf.brightness * c.conf.dec_coeff[0]
  res = c.get_cmd('dec')()
  hist = c.cmd.history['set_brightness']
  ok_(res)
  eq_(nouts, len(hist))
  for n in range(nouts):
    eq_(hist[n][1], expected)


def test_inc_ntimes():
  c = ScrDim(conf=DummyConfig(), cmd=DummyExecutor())
  nouts = len(c.cmd.connected_outputs())
  coeff = c.conf.inc_coeff[0]
  expected = c.conf.brightness * coeff * coeff * coeff
  # any value that doesn't invoke max
  ntimes = 3
  res = c.get_cmd('inc')(ntimes)
  hist = c.cmd.history['set_brightness']
  ok_(res)
  eq_(nouts * ntimes, len(hist))
  eq_(hist[4][1], expected)
  eq_(hist[5][1], expected)


def test_dec_ntimes():
  c = ScrDim(conf=DummyConfig(), cmd=DummyExecutor())
  nouts = len(c.cmd.connected_outputs())
  coeff = c.conf.dec_coeff[0]
  expected = c.conf.brightness * coeff * coeff * coeff
  # any value that doesn't invoke min
  ntimes = 3
  res = c.get_cmd('dec')(ntimes)
  hist = c.cmd.history['set_brightness']
  ok_(res)
  eq_(nouts * ntimes, len(hist))
  eq_(hist[4][1], expected)
  eq_(hist[5][1], expected)



def test_inc_max():
  c = ScrDim(conf=DummyConfig(), cmd=DummyExecutor())
  _max = c.conf[c.conf.k_max]
  c.conf.brightness = _max * 0.99
  expected = _max
  nouts = len(c.cmd.connected_outputs())
  ntimes = 2
  res = c.get_cmd('inc')(ntimes)
  hist = c.cmd.history['set_brightness']
  ok_(res)
  eq_(nouts * ntimes, len(hist))
  for n in range(nouts * ntimes):
    eq_(hist[n][1], expected)


def test_dec_min():
  c = ScrDim(conf=DummyConfig(), cmd=DummyExecutor())
  _min = c.conf[c.conf.k_min]
  c.conf.brightness = _min * 1.01
  expected = _min
  nouts = len(c.cmd.connected_outputs())
  ntimes = 2
  res = c.get_cmd('dec')(ntimes)
  hist = c.cmd.history['set_brightness']
  ok_(res)
  eq_(nouts * ntimes, len(hist))
  for n in range(nouts * ntimes):
    eq_(hist[n][1], expected)



from distutils.core import setup

setup(name='scr_dim',
    version='0.4.0',
    description='Make your screen darker when it is too bright',
    author='N.Sukegawa',
    author_email='nsukeg@gmail.com',
    url='https://github.com/nsuke/scr_dim',
    packages=['scr_dim', 'scr_dim.tests'],
    scripts=['scr-dim'],
    license='MIT',
    )



from nose.tools import *
import os
import tempfile
from scr_dim.ProcLock import ProcLock

# safe lockfile path for test
tmp_lockfile = os.path.join(tempfile.gettempdir(), tempfile.gettempprefix() + 'scr_dim_test')


@raises(Exception)
def test_inprocess_lock():
  try:
    a = ProcLock(tmp_lockfile)
    b = ProcLock(tmp_lockfile)
  except Exception as e:
    a.release()
    raise e


@raises(Exception)
def test_inprocess_lock_with():
  with ProcLock(tmp_lockfile):
    a = ProcLock(tmp_lockfile)


def test_can_lock_once():
  with ProcLock(tmp_lockfile):
    pass


def test_release_explicit():
  a = ProcLock(tmp_lockfile)
  a.release()
  with ProcLock(tmp_lockfile):
    pass


def test_release_with():
  with ProcLock(tmp_lockfile):
    pass
  with ProcLock(tmp_lockfile):
    pass



import subprocess
import re

class Executor(object):
  """actually chagnes OS settings"""
  _re = re.compile('([^\s]+) connected ')

  def connected_outputs(self):
    p = subprocess.Popen([
      'xrandr',
      '--current',
    ], stdout=subprocess.PIPE)
    o, _ = p.communicate()
    output_list = o.decode('utf-8').splitlines()
    return [
      m.group(1) for m in filter(None, map(self._re.match, output_list))
    ]

  def set_brightness(self, target_output, rate=1.0):
    return subprocess.call(['xrandr',
      '--output', target_output,
      '--brightness', '%f' % rate,
    ]) == 0

  def load_icc(self, profile_path, screen=0):
    return subprocess.call([
      'xcalib',
      '-d', ':%d' % screen,
      profile_path,
    ]) == 0
